package COM::TypeLibs::File;
#
# Copyright 2007 Aryeh Leib Taurog
# 
# This is free software. It is available under the same license as Perl.
# 
# This software is provided AS IS with no warranty as to its usability or
# fitness for any purpose. I disclaim all responsibility for any damage
# whatsoever resulting from the download or use of this software.

use warnings;
use strict;

use Carp;
use Win32::OLE;

# list of all registered type libraries

sub new
{
    my $class = shift;
    my $self = [];
    bless $self, $class;
    
    foreach my $file (@_)
    {
        $self->Add($file);
    }

    return $self;
}

sub TypeLibInfo
{
    my ($self, $typelib) = @_;

    $typelib ||= '.*';
    my @results;

    for my $lib (@$self) 
    {
        push @results, $lib    if($lib->{name} =~ /^$typelib/i);
    }

    return wantarray ? @results : $results[0];
}

sub Add
{
    my ($self, $lib) = @_;

    my $file = $lib;
    $file =~ s/(\.(?:dll|tlb|ocx))\\\d+$/$1/;

    carp "can't find $file\n" and return unless -s $file;
    $file = `cygpath -wal '$file'`;
    chomp $file;

    # this is a workaround for M$ bug instantiating TLI object
    # necessary with Win32::OLE < v 0.1707 (i.e. cygwin)
    my $sc  = Win32::OLE->new("MSScriptControl.ScriptControl")
        or carp "couldn't create Script Control. $!\n" and return; 

    $sc->{Language} = "VBScript";
    my $tli = $sc->Eval('CreateObject("TLI.TypeLibInfo")');

    # now we can get to business
    $tli->{ContainingFile} = $lib;
    return if Win32::OLE->LastError();

    push @$self,
          { clsid   => $tli->{GUID},
            name    => $tli->{Name},
            version => toHex($tli->{MajorVersion}) . "." .  toHex($tli->{MinorVersion}),
            path    => $lib };
}

sub toHex
{
    return sprintf("%lx",$_[0]);
}

1;
