package Fingerprint::TypeLib;
#
# Copyright 2007 Aryeh Leib Taurog
# 
# This is free software. It is available under the same license as Perl.
# 
# This software is provided AS IS with no warranty as to its usability or
# fitness for any purpose. I disclaim all responsibility for any damage
# whatsoever resulting from the download or use of this software.

use strict;
use warnings;

use vars '$AUTOLOAD';
use Carp;
use COM::TypeLibs::File;

sub new
{
    my ($self, $lib) = @_;
    my $class = ref($self) || $self;

    # right now we only process regular files
    my $path = $lib;
    $path =~ s/(\.(?:dll|tlb|ocx))\\\d+$/$1/;
    carp "can't find $path\n" and return undef unless -f $path;

    my $tl      = COM::TypeLibs::File->new($lib);

    my $objRef  = $tl->TypeLibInfo;

    bless $objRef, $class;
    return $objRef;
}

sub cache
{
}

sub isSame
{
    my ($self, $other) = @_;

    if (ref $other eq ref $self)
    {
        return ($self->clsid eq $other->clsid
            and $self->name  eq $other->name
            and $self->path  eq $other->path
            and $self->version eq $other->version);
    }
    elsif (my $otherObj = $self->new($other))
    {
        return $self->isSame($otherObj);
    }
}

sub AUTOLOAD
{
    my $self = shift;
    my $class = ref $self;
    if ($AUTOLOAD =~ /${class}::(clsid|name|version|path)/)
    {
        my $prop = $1;
        return $self->{$prop};
    }

    croak "Method $AUTOLOAD does not exist";
}

sub DESTROY {}

1;
