package Fingerprint::VB;
#
# Copyright 2007 Aryeh Leib Taurog
# 
# This is free software. It is available under the same license as Perl.
# 
# This software is provided AS IS with no warranty as to its usability or
# fitness for any purpose. I disclaim all responsibility for any damage
# whatsoever resulting from the download or use of this software.

use strict;
use warnings;

use vars '$AUTOLOAD';

use Carp;
use File::stat;
use Digest::MD5;
use VB::Filter;

sub new
{
    my ($self, $path) = @_;
    my $class = ref($self) || $self;

    # right now we only process regular files
    carp "can't find $path\n" and return undef unless -f $path;

    my $stat = stat $path or carp "can't stat $path: $!\n";

    my $objRef = { path  => $path,
                   size  => $stat->size,
                   mtime => $stat->mtime };

    bless $objRef, $class;
    return $objRef;
}

sub cache
{
    my $self = shift;

    $self->md5;

    return $self;
}

sub md5
{
    my $self = shift;
    return $self->{md5} if defined $self->{md5};

    my $path = $self->path;

    my $data;
    my $digest = Digest::MD5->new;

    open(FD, "<", "$path") or carp "can't open $path: $!\n";
    while ($data = getVbLine(\*FD))
    {
        $digest->add($data);
    }

    close(FD) or carp "can't close $path: $!\n";

    $self->{md5} = $digest->b64digest;
    return $self->{md5};
}

sub isSame
{
    my ($self, $other) = @_;

    if (ref $other eq ref $self)
    {
        return 1 if ($self->mtime == $other->mtime and $self->size == $other->size);
        return 1 if ($self->md5 eq $other->md5);
        return 0;
    }
    elsif (-f $other)
    {
        return $self->isSame($self->new($other));
    }
}

sub update  # stands in for isSame(), but also updates the original
{
    my ($self, $other) = @_;
    my $return = 0;

    $other ||= $self->path;

    if (ref $other eq ref $self)
    {
        return 1 if ($self->mtime == $other->mtime and $self->size == $other->size);

        $return = 1 if ($self->md5 eq $other->md5);

        %$self = %$other;
    }
    elsif (-f $other)
    {
        return $self->update($self->new($other));
    }

    return $return;
}

sub AUTOLOAD
{
    my $self = shift;
    my $class = ref $self;
    if ($AUTOLOAD =~ /${class}::(size|mtime|md5|path)/)
    {
        my $prop = $1;
        return $self->{$prop};
    }

    croak "Method $AUTOLOAD does not exist";
}

sub DESTROY {}

1;
