package VB::Code;
#
# Copyright 2007 Aryeh Leib Taurog
# 
# This is free software. It is available under the same license as Perl.
# 
# This software is provided AS IS with no warranty as to its usability or
# fitness for any purpose. I disclaim all responsibility for any damage
# whatsoever resulting from the download or use of this software.

use strict;
use warnings;

use vars '$AUTOLOAD';
use Carp;

sub new
{
    my ($class, $file) = @_;

    fixSlashes($file);

    open(FH, $file) or croak "Unable to open file $file: $!\n";
    my @text = <FH>;
    close FH or croak "Error closing file $file: $!\n";

    my $objRef = {  _file => $file,
                    _text => \@text }; 

    bless $objRef, $class;
    return $objRef;
}

sub Code
{
    my $self = shift;
    $self->number();

    my @numbers = $self->_numbers();
    return @numbers unless @_;
    return @numbers[@_];
}

sub number
{
    my $self = shift;
    return if defined $self->{_numbers};

    my @numbers;
    my $state = ($self->_file() =~ /\.bas$/) ? 2 : 0;
    my $depth = 0;

    foreach ($self->_text())
    {
        if (/^\s*Begin/i and $state <= 1)
        {
            $state = 1;
            $depth++;
        }
        elsif (/^\s*End/i and $state == 1)
        {
            $depth--;
            $state = 2 if $depth == 0;
        }
        elsif (!/^Attribute / and $state == 2)
        {
            push @numbers, $_;
        }
    }
    $self->{_numbers} = \@numbers;
}

sub fixSlashes
{
    map s"\\"/"g, @_;
}

sub AUTOLOAD
{
    my $self = shift;
    my $class = ref $self;
#   if ($AUTOLOAD =~ /${class}::(Type|Name|ExeName32|Description)/)
#   {
#       my $prop = $1;
#       return $self->{"_$prop"} if defined $self->{"_$prop"};

#       my ($dataline) = grep /^$prop="(.*)"/, $self->_text();
#       return "" unless defined $dataline;
#       $dataline =~ /$prop="(.*)"/;
#       $self->{"_$prop"} = $1;
#       return $self->{"_$prop"};
#   }

    if ($AUTOLOAD =~ /::(_\w+)$/)
    {
        my $ptr = $self->{$1};
        return '' unless defined $ptr;

        my $type = ref $ptr;
        if    ($type eq 'SCALAR') { return ${$ptr} }
        elsif ($type eq 'ARRAY')  { return @{$ptr} }
        elsif ($type eq 'HASH')   { return %{$ptr} }
        else                      { return $ptr    }
    }

    croak "Method $AUTOLOAD does not exist";
}

sub DESTROY {}

1;
