#!/usr/bin/perl -w
#
# Copyright 2007 Aryeh Leib Taurog
# 
# This is free software. It is available under the same license as Perl.
# 
# This software is provided AS IS with no warranty as to its usability or
# fitness for any purpose. I disclaim all responsibility for any damage
# whatsoever resulting from the download or use of this software.
#
use strict;
use File::Basename;
use COM::TypeLibs::Registry;
use COM::TypeLibs::File;

foreach my $file (@ARGV)
{
    process_file($file);
}

sub process_file
{
    my $filePath = shift;
    my $basePath = dirname($filePath);

    open FH, "<", $filePath or die "can't open $filePath for reading: $!\n";
    my @content = <FH>;
    close FH or die "error closing $filePath after read: $!\n";

    if (!@content)
    {
        print "got no content in $filePath\n";
        return;
    }

    my @updatedContent = map update_reference($_ , $basePath) , @content;

    if (!@updatedContent)
    {
        print "error updating $filePath\n";
        return;
    }

    open FH, ">", $filePath or die "can't open $filePath for writing: $!\n";
    print FH @updatedContent;
    close FH or die "error closing $filePath after writing: $!\n";
}


sub update_reference
{
    my $line = shift;
    my $path = shift;

    my $h   = '[0-9A-F]';
    my $ch  = '[0-9A-F-]';

    if ($line =~ /^Reference=\*\\G(\{$ch{36}\})#($h+)\.($h+)#0#(.+)#(.+)$/i)
    {
        my $refClsid = $1;
        my $refMajor = $2;
        my $refMinor = $3;
        my $refPath  = $4;
        my $refName  = $5;
        $refName =~ s/\s*$//g;

        #$refPath =~ s"\\"/"g;
        my $fileTliObj  = COM::TypeLibs::File->new("$path/$refPath");
        my @fileInfo    = $fileTliObj->TypeLibInfo();
        die "$refPath is not a typelib\n" unless @fileInfo;

        if (@fileInfo > 1)
        {
            warn "$refName reference contains multiple libraries";
            return $line;
        }

        my $fileInfo = $fileInfo[0];
        my $fileClsid = $fileInfo->{clsid};
        my $fileVersion = $fileInfo->{version};
        my $fileName = $fileInfo->{name};

        if ($refClsid ne $fileClsid or "$refMajor.$refMinor" ne $fileVersion)
        {
            print "updating reference information for $refName\n";
            $line = "Reference=*\\G$fileClsid#$fileVersion#0#$refPath#$refName\r\n";
        }
        else
        {
            print "reference for $refName is up to date\n";
        }

        my $name = $fileInfo->{name};
        return $line unless $name;
        my $regTliObj   = COM::TypeLibs::Registry->new();

        my @regInfo     = $regTliObj->TypeLibInfo($name);
        my @regMatch    = grep $_->{clsid} eq $fileClsid && $_->{version} eq $fileVersion, @regInfo;
        print "current clsid isn't registered: $fileName $fileClsid#$fileVersion\n" unless @regMatch;
    }

    return $line;
}

